#!/bin/sh +x

#exec 2> /tmp/err

cd `dirname $0`

Root="/tftpboot"
Hostname=`hostname`
HostnameIP=`host $Hostname 2> /dev/null | sed "s/.*address //"`
[ -z "$HostnameIP" ] && HostnameIP=`grep $Hostname /etc/hosts | awk '{ print $1}'`
Kernel="bzImage-nfs-root-2-4-18"
#Kernel="2.2.16/bzImage-nfs-root-2-2-16"
Kernel="2.4.17/bzImage-term"
ServeurX="$HostnameIP"
Mouse="ttyS0"
Disquette="/dev/fd0"
PointMontage="/mnt"
ConfigIpPnp="DHCP"

[ `id -u` -ne 0 ] && echo >&2 "Il faut etre root pour executer `basename $0`" && exit 1

usage ()
{
    echo >&2 "
Usage : `basename $0` -i <adresse IP client> -m <adresse MAC client>
                    -H <hostname client> [ -c masque de reseau ] [ -g Gatway ]
                    [ -r root ] [ -k kernel ] [ -u ] [ -nf ]
                    [ -X <IP du serveur X> ] [ -s <souris> ]
Disquette
PointMontage
--lilo
--lilo-bmp
--lilo-penguins
--lilo-lpp
--lpp ?
--syslinux
--dhcp
--bootp
--rarp
--net-boot
--strip-debug
--strip-all
--append <append>"
}
bold=`tput smso`
offbold=`tput rmso`
B=`echo -e "bold" | tput -S` # Bold
N=`tput sgr0`                      # Normal
clear="$N `tput clear`"               # Clear Screen
home="$N `tput home`"                # Move cursor to top left corner

echo "$clear"

NbCols=`tput cols`
NbLignes=0

Progression()
{
    case "$1" in
        ?) Pct="   $1" ;;
        ??) Pct="  $1" ;;
        ???) Pct=" $1" ;;
    esac
    tput cup $NbLignes 0
    echo "$2"
    tput cup $NbLignes `expr $NbCols - 10`
    echo "[$Pct % ]"
    NbLignes=`expr $NbLignes + 1`
}

while [ "$1" ]
do
    case "$1" in
        -i) AdrIP=$2 ; shift ;;
        -m) AdrMac=$2 ; shift ;;
        -H) HostnameClient=$2 ; shift ;;
        -r) Root=$2 ; shift ;;
        -c) Masque=$2 ; shift ;;
        -g) Gatway=$2 ; shift ;;
        -k) Kernel=$2 ; shift ;;
        -u) User="y" ;;
        -nf) NoFloppy="y" ;;
        -X) ServeurX="$2" ; shift ;;
        -s) Mouse="$2" ; shift ;;
        --append) Append="$1" ; shift ;;
        --lilo) Boot="lilo" ;;
        --lilo-bmp) Boot="lilo-bmp" ;;
        --lilo-penguins) Boot="lilo-penguins" ;;
        --lilo-lpp) [ -z "$Boot" ] && Boot="lilo"
                    LPP="y"
                    Append="$Append console=/dev/tty2 CONSOLE=/dev/tty2"
                    Kernel="2.4.17/vmlinuz-lpp"
                    #[ -z "$Kernel" ] && Kernel="bzImage-nfs-root-lpp-2-4-17"
                    ;;
        --lpp) Boot="lpp"
               [ -z "$Kernel" ] && Kernel="bzImage-nfs-root-lpp-2-4-17"
               ;;
        --syslinux) Boot="syslinux" ;;
        --dhcp) ConfigIpPnp="DHCP" ; Boot="net" ;;
        --bootp) ConfigIpPnp="BOOTP" ; Boot="net" ;;
        --rarp) ConfigIpPnp="RARP" ; Boot="net" ;;
        --net-boot) Boot="net" ;;
        -h) usage ; exit 0 ;;
        *)  echo >&2 "Argument $1 non valide" ; usage ; exit 1 ;;
    esac
    shift
done

[ -z "$AdrIP" ] && echo "Set IP address" && usage && exit 1
[ -z "$HostnameClient" ] && HostnameClient="term`echo $AdrIP | cut -d. -f4`"
[ -z "$AdrMac" ] && ping -c 1 $AdrIP > /dev/null && AdrMac=`arp -n $AdrIP | grep $AdrIP | awk '{print $3}'`
[ -z "$AdrMac" ] && echo "Set MAC address" && usage && exit 1

#export Root AdrIP

umount $PointMontage 2> /dev/null

Progression 0 "$Root/$AdrIP"
for rep in $Root/$AdrIP
do
    [ -d $rep ] && rm -rf $rep*
    mkdir -p $rep
done

[ -d /users ] && Home=/users
[ -d /home ] && Home=/home

#
# Creation du mini systeme root avant de tout monter par NFS.
#

Progression 4 "basic reps"
for rep in bin etc home lib mnt proc sbin tmp usr var/lib
do
    mkdir -p $Root/$AdrIP/$rep
done

Progression 9 "/bin"
for prog in /bin/mount /bin/hostname /bin/sed /bin/mv /bin/ln /bin/loadkeys
do
    cp $prog $Root/$AdrIP/bin
done

Progression 13 "/dev"
cp -a /dev/ $Root/$AdrIP

Progression 15 "/sbin"
for prog in /sbin/init /sbin/ifconfig /sbin/route
do
    cp $prog $Root/$AdrIP/sbin
done

Progression 18 "libs"
cp -a /lib/libc.so* /lib/ld* $Root/$AdrIP/lib

#
# Du menage...
#

Progression 20 "/var"
for rep in /var/lock /var/run
do
    [ ! -d $Root/$AdrIP/$rep ] && mkdir -p $Root/$AdrIP/$rep
done

touch $Root/$AdrIP/etc/mtab

Progression 24 "PAM"
cp -a /etc/pam* /etc/security /etc/securetty $Root/$AdrIP/etc
cp -a /lib/security/ $Root/$AdrIP/lib

Progression 27 "misc files"
cp /etc/services /etc/protocols $Root/$AdrIP/etc

Progression 30 "defkeymap.map"
dumpkeys > $Root/$AdrIP/etc/defkeymap.map

#
# Les fichiers non prits sur le systeme
#

Progression 32 "term fics"
for fic in `find term -type f`
do
    ACopier=`echo $fic | sed "s;term;\$Root/\$AdrIP;"`
    [ ! -d `dirname $ACopier` ] && mkdir -p `dirname $ACopier`

    [ -f $ACopier ] && rm -f $ACopier
    cp $fic $ACopier
done

for fic in `find term -type l`
do
    ACopier=`echo $fic | sed "s;term;\$Root/\$AdrIP;"`
    cp -pR $fic $ACopier
done

echo "NFS-Root" > $Root/$AdrIP/fastboot

#
# Le programme servant a se connecter n'est peut etre pas le meme que
# celui utilise par monLinux (getty). Ce n'est pas bien grave, on fait
# un lien.
#

Progression 44 "/sbin/init & /etc/inittab"
ProgInit=`grep "^1:" /etc/inittab | cut -d: -f4 | sed "s/ .*//"`
cp $ProgInit $Root/$AdrIP/$ProgInit

#
# Les fichiers de demarrage est copie.
#

grep "^[^#]*getty" /etc/inittab >> $Root/$AdrIP/etc/inittab

[ ! -d $Root/$AdrIP/etc/init.d ] && mkdir -p $Root/$AdrIP/etc/init.d

cp /etc/hosts $Root/$AdrIP/etc/hosts

Classe=`echo $AdrIP | cut -d. -f1`

if [ -z "$Masque" ]
then
    [ $Classe -le 127 ] && Masque="255.0.0.0"
    [ $Classe -ge 128 -a $Classe -le 191 ] && Masque="255.255.0.0"
    [ $Classe -ge 192 -a $Classe -le 1223 ] && Masque="255.255.255.0"
fi

if echo "$Masque" | egrep "[^0]+\.0\.0\.0$" > /dev/null
then
    Network="`echo $AdrIP | cut -d. -f1`.0.0.0"
    [ -z "$Gatway" ] && Gatway="`echo $AdrIP | cut -d. -f1`.0.0.1"
    Broadcast="`echo $AdrIP | cut -d. -f1`.255.255.255"
elif echo "$Masque" | egrep "[^0]+\.[^0]+\.0\.0$" > /dev/null
then
    Network="`echo $AdrIP | cut -d. -f1`.`echo $AdrIP | cut -d. -f2`.0.0"
    [ -z "$Gatway" ] && Gatway="`echo $AdrIP | cut -d. -f1`.`echo $AdrIP | cut -d. -f2`.0.1"
    Broadcast="`echo $AdrIP | cut -d. -f1`.`echo $AdrIP | cut -d. -f2`.255.255"
elif echo "$Masque" | egrep "[^0]+\.[^0]+\.[^0]+\.0$" > /dev/null
then
    Network="`echo $AdrIP | cut -d. -f1`.`echo $AdrIP | cut -d. -f2`.`echo $AdrIP | cut -d. -f3`.0"
    [ -z "$Gatway" ] && Gatway="`echo $AdrIP | cut -d. -f1`.`echo $AdrIP | cut -d. -f2`.`echo $AdrIP | cut -d. -f3`.1"
    Broadcast="`echo $AdrIP | cut -d. -f1`.`echo $AdrIP | cut -d. -f2`.`echo $AdrIP | cut -d. -f3`.255"
else
    Network="** A CHANGER **"
    [ -z "$Gatway" ] && Gatway="** A CHANGER **"
    Broadcast="** A CHANGER **"
fi

DNS1=`grep nameserver /etc/resolv.conf | head -n 1 | sed "s/nameserver //"`
DNS2=`grep nameserver /etc/resolv.conf | tail -n 1 | sed "s/nameserver //"`

cat << EOF > $Root/$AdrIP/etc/init.d/network
HOSTNAME=$HostnameClient
DOMAINNAME=`hostname -d`
DEVICE=eth0
IPADDR=$AdrIP
NETWORK=$Network
GATEWAY=$Gatway
NETMASK=$Masque
BROADCAST=$Broadcast
DNS1=$DNS1
DNS2=$DNS2
EOF


Progression 59 "$Root/$AdrIP/etc/fstab"
cat << EOF > $Root/$AdrIP/etc/fstab
/dev/root                      /               nfs     defaults        1 1
none                           /proc           proc    defaults        0 0
$HostnameIP:/bin                   /bin            nfs     defaults        1 1
$HostnameIP:/usr                   /usr            nfs     defaults        1 1
$HostnameIP:/sbin                  /sbin           nfs     defaults        1 1
#$HostnameIP:/home                  /home           nfs     defaults        1 1
$HostnameIP:/lib                   /lib            nfs     defaults        1 1
$HostnameIP:$Home                  $Home          nfs     defaults        1 1
$HostnameIP:/var/lib       /var/lib         nfs     defaults        1 1
EOF

Progression 62 "/etc/passwd, ..."
if [ "$User" = "y" ]
then
    cp -f /etc/passwd /etc/group /etc/shadow $Root/$AdrIP/etc
else
    for fic in /etc/passwd /etc/shadow /etc/group
    do
        grep "^root:" $fic > $Root/$AdrIP/$fic
    done
fi

chmod 644 $Root/$AdrIP/etc/passwd $Root/$AdrIP/etc/group
chmod 600 $Root/$AdrIP/etc/shadow

DirRoot=`grep "^root:" /etc/passwd | cut -d: -f6`
[ ! -d $Root/$AdrIP/$DirRoot ] && mkdir -p $Root/$AdrIP/$DirRoot

#
# X !!!
#

if [ -n "$ServeurX" ]
then
    Progression 70 "X"
    (cd $Root/$AdrIP/dev && ln -sf $Mouse mouse)

    echo "x:1:respawn:/usr/X11R6/bin/XF86_SVGA -indirect $ServeurX" >> $Root/$AdrIP/etc/inittab
else
    Progression 70 "X (pas utilise)"
fi

#
# Configuration du serveur
#

Progression 73 "/etc/hosts"
sed "/$HostnameClient/d" < /etc/hosts > /etc/hosts.old
echo "$AdrIP    $HostnameClient.`hostname -d`    $HostnameClient" >> /etc/hosts.old
mv -f /etc/hosts.old /etc/hosts

Progression 93 "/etc/exports"
grep -v "$Root/$AdrIP" < /etc/exports > /etc/exports.old
echo "$Root/$AdrIP          $HostnameClient(rw,no_root_squash)" >> /etc/exports.old
mv -f /etc/exports.old /etc/exports

for rep in /bin /var/lib /usr /sbin /lib /home
do
    AExporter=`grep "^ *$rep" /etc/exports | cut -d " " -f2- | tr -s ' \n' ' '`
    grep -v "^ *$rep" < /etc/exports > /etc/exports.old
    if echo "$AExporter" | grep "$HostnameClient" > /dev/null
    then :
    else
        [ "$rep" = "/home" ] && AExporter="$AExporter $HostnameClient(rw,no_root_squash)" || AExporter="$AExporter $HostnameClient(ro,no_root_squash)"
    fi
    echo "$rep    $AExporter" >> /etc/exports.old
    mv -f /etc/exports.old /etc/exports
done

#
# MAJ de /etc/hosts.allow...
#

sed -e "/in.tftpd: *$Network.$Masque/,/portmap: *$Network.$Masque/d" \
    -e "/bootpd: *0.0.0.0/d" < /etc/hosts.allow > /etc/hosts.allow.old

cat << EOF >> /etc/hosts.allow.old
bootpd:   0.0.0.0
in.tftpd: $Network/$Masque
portmap:  $Network/$Masque
EOF

mv -f /etc/hosts.allow.old /etc/hosts.allow

if [ "$ConfigIpPnp" = "RARP" ]
then

#
# Mise a jour de rarpd
#

    /sbin/rarp -s $AdrIP $AdrMac
## MAJ fic boot local

elif [ "$ConfigIpPnp" = "BOOTP" ]
then

#
# Mise a jour de bootpd
#

    sed "/^ *[^#]*$HostnameClient:/,/^$/d" < /etc/bootptab | uniq > /etc/bootptab.old
    cat << EOF >> /etc/bootptab.old
$HostnameClient:ht=ethernet:ha=`echo $AdrMac | tr -d ":"`:ip=$AdrIP:td=$Root:hd=/:bf=$Kernel:
EOF
    mv -f /etc/bootptab.old /etc/bootptab
    cp $Kernel $Root
    sed "s;^ *#* *\(bootps.*-c\) \(.*\);\1 $Root;" < /etc/inetd.conf > /etc/inetd.conf.old
    mv -f /etc/inetd.conf.old /etc/inetd.conf
    killall -HUP inetd

elif [ "$ConfigIpPnp" = "DHCP" ]
then

#
# Mise a jour de DHCPD
#

    sed "/^ *[^#]*host $HostnameClient/,/}/d" < /etc/dhcpd.conf | uniq > /etc/dhcpd.conf.old

    cat << EOF >> /etc/dhcpd.conf.old
host $HostnameClient {
    hardware ethernet $AdrMac;
    fixed-address $AdrIP;
EOF

    [ "$Boot" = "net" ] && cat << EOF >> /etc/dhcpd.conf.old
    #filename              "/tftpboot/lts/vmlinuz-2.4.18-ltsp-1";
    #filename              "/$HostnameClient.boot";
    filename              "/pxelinux.0";
    option vendor-encapsulated-options 09:0f:80:00:0c:4e:65:74:77:6f:72:6b:20:62:6f:6f:74:0a:07:00:50:72:6f:6d:70:74:06:01:02:08:03:80:00:00:47:04:80:00:00:00:ff;
EOF

    echo "}" >> /etc/dhcpd.conf.old

    mv -f /etc/dhcpd.conf.old /etc/dhcpd.conf
fi

Progression 100 "END"

if [ `grep -ic "^ *[^#]*$AdrMac" /etc/dhcpd.conf` -gt 1 ]
then
    cat << EOF >&2

** ATTENTION : plusieurs definitions pour l'adresse MAC $AdrMac
dans le fichier /etc/dhcpd.conf. Il n'en faut qu'une !
EOF
fi


#
# Generation de la disquette de boot...
#

[ -e /dev/nfsroot ] || mknod /dev/nfsroot b 0 255

echo

if [ "$NoFloppy" != "y" ]
then
    if [ "$Boot" = "net" ]
    then :
    else
        echo "Mettre une disquette (kernel $Kernel)"
        read
    fi

    case "$Boot" in
        lilo*) 
              cat lilo-bmp/fd.img > $Disquette
#              offset=`expr \`ls -Ls $Kernel | awk '{print $1}'\` + 100`
#              yes | mke2fs -N 24 -m 0 $Disquette 1440000 > /dev/null 2>&1
              #yes | mke2fs -N 24 -m 0 $Disquette $offset > /dev/null 2>&1

              mount -o dev $Disquette $PointMontage
              rm -rf $PointMontage/lost+found
              mkdir $PointMontage/{boot,dev,etc}

              cp -R /dev/{null,fd0,ram0} $PointMontage/dev
              cp /boot/boot.b $PointMontage/boot
              cp $Kernel $PointMontage/vmlinuz
              cp -a /dev/nfsroot $PointMontage/dev

        cat << EOF > $PointMontage/etc/lilo.conf
prompt
#timeout = 100
boot      = $Disquette
EOF

        if [ "$Boot" = "lilo-bmp" ]
        then
            Lilo=`./lilo-bmp/lilo-ok.sh`
            cp lilo-bmp/boot-bmp.b $PointMontage/boot/boot.b
            cp lilo-bmp/ins64a.bmp $PointMontage/boot/
            cat << EOF >> $PointMontage/etc/lilo.conf
install = /boot/boot.b
bitmap = /boot/ins64a.bmp
bmp-colors = 14,11,,15,9,0
bmp-table = 21,287p,2,4,175p
EOF
        elif [ "$Boot" = "lilo-penguins" ]
        then
            Lilo=lilo-bmp/lilo
            #Lilo=`./lilo-bmp/lilo-ok.sh`
            cp lilo-bmp/boot-menu.b $PointMontage/boot/boot.b
            cp lilo-bmp/penguins.boot $PointMontage/boot/
            cat << EOF >> $PointMontage/etc/lilo.conf
install = /boot/boot.b
message = /boot/penguins.boot
EOF
        fi

        if [ -n "$Append" ]
        then
            cat << EOF >> $PointMontage/etc/lilo.conf
append = "$Append"
EOF
        fi

        if [ "$LPP" = "y" ]
        then
cat << EOF >> $PointMontage/etc/lilo.conf
#vga=ask
vga=0x301
EOF
        fi

        cat << EOF >> $PointMontage/etc/lilo.conf
map       = /boot/map
read-write
backup    = /dev/null
image     = /vmlinuz
label     = linux-term
root      = /dev/nfsroot
EOF

        [ -z "$Lilo" ] && Lilo=lilo
        $Lilo -v -C /etc/lilo.conf -r $PointMontage
        umount $PointMontage
        ;;
        syslinux) syslinux -s $Disquette
                  mount -o dev $Disquette $PointMontage
                  cp $Kernel $PointMontage/linux
                  cat << EOF > $PointMontage/syslinux.cfg
default linux
   append init=/sbin/init root=/dev/nfsroot

   prompt 1
#   timeout 30
   readinfo 2

EOF
        #ip=:195.251.160.10:195.251.160.254:255.255.255.0:::'bootp'
        #nfsroot=195.251.160.10:/usr/local/linux/ws/\%s vga=0x318
           umount $PointMontage
           ;;
        lpp) vidmode $Kernel -1
             cat $Kernel > /dev/fd0
             ;;
        net) cp $Kernel $Root/$HostnameClient.boot
             cp pxestuff/pxelinux.0 $Root
             mkdir -p $Root/pxelinux.cfg
             for i in `echo $AdrIP | tr "." " "`
             do
                 FicPxelinux="$FicPxelinux`./dec2hexa.sh $i`"
             done
             cat << EOF > $Root/pxelinux.cfg/$FicPxelinux
prompt=0
label linux
        kernel $HostnameClient.boot
EOF
# append init=/linuxrc rw root=/dev/ram0 initrd=initrd-...
             ;;
#./mknbi $Kernel $Root/$HostnameClient.boot ;;
        *) rdev $Kernel /dev/nfsroot
           cat $Kernel > /dev/fd0
           ;;
    esac

fi

if [ -x /etc/init.d/netfs ]
then
    /etc/init.d/netfs reload && NFS="OK"      # RedHat
elif [ -x /sbin/init.d/nfsserver ]
then
    /sbin/init.d/nfsserver reload && NFS="OK" # SuSE
elif [ -x /etc/rc.d/rc.inet2 ]
then
    /etc/rc.d/rc.inet2 && NFS="OK"            # Slackware
elif [ -x /etc/init.d/nfs-server ]
then
    /etc/init.d/nfs-server reload && NFS="OK" # Debian
elif [ -n "`which exportfs`" ]
then
    `which exportfs` -r && NFS="OK"
elif [ -n "`which kexportfs`" ]
then
    `which kexportfs` -r && NFS="OK"
fi

[ "$NFS" != "OK" ] && echo "Relancer le serveur NFS"

if [ -x /etc/rc.d/dhcp ] # SuSE
then
    /etc/rc.d/dhcp reload
    DHCP="OK"
elif [ -x /etc/init.d/dhcp ] # Debian
then
    /etc/init.d/dhcp reload
    DHCP="OK"
elif [ -x /etc/rc.d/init.d/dhcp ] # RedHat
then
    /etc/rc.d/init.d/dhcp reload
    DHCP="OK"
fi
[ "$DHCP" != "OK" ] && echo "Relancer le serveur DHCP"

Banner="************************************************"

cat << EOF

$Banner
  Editer (eventuellement) le fichier :
    $Root/$AdrIP/etc/init.d/network
  pour configurer le reseau pour la station.

  Ce fichier contient :

`sed "s/^/                  /" < $Root/$AdrIP/etc/init.d/network`

  Pour configurer XFree, copier le fichier de configuration dans :
    $Root/$AdrIP/etc/XF86Config

$Banner

EOF

