#!/bin/sh

#
# majX.sh - configure le serveur XFree 3
#
# Description :
# majX.sh configure le serveur XFree86 version 3 a partir d'un fichier
# modele. Les modifications apportees a ce modele sont le type de souris,
# ...
#

cd `dirname $0`

if [ $# -eq 1 ]
then
    if [ "$1" = "--Xdialog" ]
    then
        dialog=y
    else
        echo >&2 "Erreur : argument $1 non valide"
        exit 1
    fi
fi

rm -f /tmp/probe-mouse

MajFicConf ()
{
    cp $FicXFree /etc/XF86Config
    case "$1" in
        psaux) Protocole="PS/2" ;;
        ttyS*) Protocole="Microsoft" ;;
        *) Protocole=Busmouse ;;
    esac
    sed "s;Protocol.*@@MOUSE@@;Protocol \"$Protocole;" < /etc/XF86Config > /etc/XF86Config.old
    mv -f /etc/XF86Config.old /etc/XF86Config
    [ -f /tmp/probe-mouse ] && Periph=`cat /tmp/probe-mouse`
    ln -sf /dev/$Periph /dev/mouse
}

mdetect > /tmp/probe-mouse || ./probe-mouse.sh
[ -f /tmp/probe-mouse ] && Periph=`cat /tmp/probe-mouse`

if [ -f /etc/X11/XF86Config.orig ]
then
    FicXFree=/etc/X11/XF86Config.orig
elif [ -f ./XF86Config.orig ]
then
    FicXFree=./XF86Config.orig
else
    echo >&2 "Erreur : fichier XF86Config.orig non trouve"
    exit 1
fi

if [ -n "$Periph" ]
then
    MajFicConf $Periph
    echo $Periph > /etc/mouse
else
    X=`which X`
    if [ $? -ne 0 ]
    then
        X=""
        for x in `whereis X | sed "s/.*: //"`
        do
            [ -L $x ] && [ -e `ls -L $x` ] && X=$x && break || continue
#            echo $x | grep "/bin/X$" > /dev/null && X=$x && break
        done
        if [ -z "$X" ] # On a pas trouve au dessus
        then
            X=`grep "XF86_" /etc/inittab | cut -d: -f4 | cut -d " " -f1`
        fi
    fi
    for x in `which xmessage` `whereis xmessage | sed "s/.*: //"`
    do
        [ -L $x ] && continue
        echo $x | grep "/bin/xmessage$" > /dev/null && xmessage=$x && break
    done
    for Periph in psaux ttyS0 ttyS1
    do
        MajFicConf $Periph
        if [ -n "$X" ]
        then
            if [ "$dialog" = "y" ]
            then
                for x in `whereis Xdialog | sed "s/.*: //"`
                do
                    [ -L $x ] && continue
                    echo $x | grep "/bin/Xdialog$" > /dev/null && Xdialog=$x && break
                done
            fi
            export DISPLAY=:0
            if [ -n "$Xdialog" ]
            then
                $X &
                PID_X=$!
                /usr/X11R6/bin/xsetroot -solid blue -cursor_name left_ptr &
                #/usr/X11R6/bin/xsetroot -solid blue -cursor_name left_ptr -display :0 &
                #DISPLAY=:0 Xdialog --yesno "Is the mice OK ?
                Xdialog --yesno "Is the mice OK ?
Press [Up] key and [Enter] if not" 7 40
                retour=$?
                kill -TERM $PID_X
                [ $retour -eq 0 ] && echo $Periph > /etc/mouse && exit 0
            else
                echo "Mouse not found. Trying $Periph"
                echo "XF86 test. If this doesn't work, please shut down X server
with <Ctrl-Alt-Backspace>.

Press any key to continue"
                read t
                $X 2> /dev/null &
                PID_X=$!
                /usr/X11R6/bin/xsetroot -solid blue -cursor_name left_ptr 2> /dev/null &
                $xmessage -center -timeout 10 "Try mouse and press OK
If you cannot press any button, wait 10 sec..." 2> /dev/null
                [ $? -eq 0 ] && kill -TERM $PID_X
                echo "
Is the mice OK ([y]/n) ?"
                read a
                [ "$a" = "y" -o -z "$a" ] && echo $Periph > /etc/mouse && exit 0
            fi
        else
            echo >&2 "Erreur : X non trouve"
            exit 1
        fi
    done
fi

rm -f /tmp/probe-mouse
